--	GameState.lua - Main script run by LuaGameState.cpp.
--  ----------------------------------------------------------------------------
--	This file is part of 'NiallsAVLib', base code for any kind of audiovisual
--	apps.
--	Copyright (C) 2013  Niall Moody
--	
--	This program is free software: you can redistribute it and/or modify
--	it under the terms of the GNU General Public License as published by
--	the Free Software Foundation, either version 3 of the License, or
--	(at your option) any later version.
--
--	This program is distributed in the hope that it will be useful,
--	but WITHOUT ANY WARRANTY; without even the implied warranty of
--	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--	GNU General Public License for more details.
--
--	You should have received a copy of the GNU General Public License
--	along with this program.  If not, see <http://www.gnu.org/licenses/>.
--	----------------------------------------------------------------------------

--------------------------------------------------------------------------------
--	Our 6 moon images.
images = {"Moon1", "Moon2", "Moon3", "Moon4", "Moon5", "Moon6"}
--	The current moon image.
currentImage = 1
--	The previous moon image.
prevImage = 6
--	Used to animate the moon.
imageTick = 1

--	Our 6 sounds.
sounds = {"voice00",
		  "voice01",
		  "voice02",
		  "voice03",
		  "voice04",
		  "voice05",
		  "voice06",
		  "voice07",
		  "voice08",
		  "voice09"}
--	The number of loops we've triggered.
numLoops = 0

--	The current mouse position.
mousePos = TwoFloats()
--	The last mouse position.
lastPos = TwoFloats()
--	The trigger points.
triggerPoints = {}

--	Used to toggle the overload thingy.
overloadToggle = false
--	Used to overload the alpha blending.
overload = 0.0

--	Used to toggle the scale up fade out thingy.
scaleToggle = false
--	Used to fade in/out the scale up fade out thingy.
scaleFade = 0.0
--	Used to create a scale up fade out tunnel thing.
scale = 0.0
--	Used to create a scale up fade out tunnel thing.
scale2 = 1.0
--	Used to create a scale up fade out tunnel thing.
scale3 = 2.0
--	Used to create a scale up fade out tunnel thing.
scale4 = 3.0

--	Used to toggle the colour cycling thing.
cycleToggle = false
--	The (HSL) colour for the colour cycling.
cycleColour = ThreeFloats(0.0, 0.8, 0.7)
--	Used to fade in/out the colour cycling.
cycleFade = 0.0

--	Used to toggle the granulators.
granulatorsToggle = false
--	Used to toggle the comb filters.
combFiltersToggle = false

--	Our copy of the screen size.
drawingSize = TwoFloats()

--------------------------------------------------------------------------------
--	Called when the state is first initialised.
function initialise(drawSize)
	drawingSize = drawSize

	mousePos = drawingSize*0.5
	lastPos = mousePos

	for i=1,6 do
		triggerPoints[i] = TwoFloats(Random.getGlobal():getRandomFloat() * drawingSize.x,
									 Random.getGlobal():getRandomFloat() * drawingSize.y)
	end
end

--------------------------------------------------------------------------------
--	Used to update game logic etc.
function update(sound)
	imageTick = imageTick + 1
	randVal = Random.getGlobal():getRandomFloat()
	effect = false

	if (imageTick > 64) then
		imageTick = 1

		prevImage = currentImage
		currentImage = currentImage + 1
		if (currentImage > 6) then
			currentImage = 1
		end
	end

	if (lastPos ~= mousePos) then
		lastPos = mousePos

		if (numLoops > 12) then
			effect = true
		elseif (numLoops < 24) then

			for i=1,6 do
				if (mousePos:getDistance(triggerPoints[i]) < 64.0) then
					soundIndex = Random.getGlobal():getRandomInt(#sounds) + 1

					sound:triggerLoop(sounds[soundIndex], 1.0, 0.5, true)

					--table.remove(sounds, i)

					triggerPoints[i] = TwoFloats(Random.getGlobal():getRandomFloat() * drawingSize.x,
												 Random.getGlobal():getRandomFloat() * drawingSize.y)

					numLoops = numLoops + 1
					break
				end
			end
		else
			effect = true
		end

		if effect and (Random.getGlobal():getRandomFloat() > 0.75) then
			randVal = Random.getGlobal():getRandomInt(10)
			
			if (randVal == 0) then
				if (overload <= 0.0) or (overload >= 3.0) then
					overloadToggle = not overloadToggle
				end
			elseif (randVal == 1) then
				if (scaleFade <= 0.0) or (scaleFade >= 1.0) then
					scaleToggle = not scaleToggle
				end
			elseif (randVal == 2) then
				if (cycleFade <= 0.0) or (cycleFade >= 1.0) then
					cycleToggle = not cycleToggle
				end
			elseif (randVal == 3) then
				granulatorsToggle = not granulatorsToggle
				sound:toggleGranulators(granulatorsToggle)
			elseif (randVal == 4) then
				combFiltersToggle = not combFiltersToggle
				sound:toggleCombFilters(combFiltersToggle)
			elseif (randVal > 7) then
				if overloadToggle and ((overload <= 0.0) or (overload >= 3.0)) then
					overloadToggle = false
				elseif scaleToggle and ((scaleFade <= 0.0) or (scaleFade >= 1.0)) then
					scaleToggle = false
				elseif cycleToggle and ((cycleFade <= 0.0) or (cycleFade >= 1.0)) then
					cycleToggle = false
				elseif granulatorsToggle then
					granulatorsToggle = false
					sound:toggleGranulators(granulatorsToggle)
				elseif combFiltersToggle then
					combFiltersToggle = false
					sound:toggleCombFilters(combFiltersToggle)
				end
			end
		end
	end

	if overloadToggle and (overload < 3.0) then
		overload = overload + 0.05
	elseif not overloadToggle and (overload > 0.0) then
		overload = overload - 0.05
	end

	if scaleToggle then
		if (scaleFade < 1.0) then
			scaleFade = scaleFade + 0.01
		end

		scale = scale + 0.01
		if (scale >= 4.0) then
			scale = 0.0
		end
		scale2 = scale2 + 0.01
		if (scale2 >= 4.0) then
			scale2 = 0.0
		end
		scale3 = scale3 + 0.01
		if (scale3 >= 4.0) then
			scale3 = 0.0
		end
		scale4 = scale4 + 0.01
		if (scale4 >= 4.0) then
			scale4 = 0.0
		end
	elseif (scaleFade > 0.0) then
		scaleFade = scaleFade - 0.01
	end

	if cycleToggle then
		if (cycleFade < 1.0) then
			cycleFade = cycleFade + 0.01
		end

		cycleColour.x = cycleColour.x + 0.005
		if (cycleColour.x > 1.0) then
			cycleColour.x = 0.0
		end
	elseif (cycleFade > 0.0) then
		cycleFade = cycleFade - 0.01
	end
end

--------------------------------------------------------------------------------
--	Draws the game.
function draw(d)
	pos = TwoFloats()
	fade = ((imageTick-1.0)/63.0) + overload
	col = ThreeFloats(1.0)

	if cycleToggle then
		col = col + ((hslToRgb(cycleColour) - col) * cycleFade)
	elseif (cycleFade > 0.0) then
		col = col + ((hslToRgb(cycleColour) - col) * cycleFade)
	end
	
	pos = (drawingSize * 0.5) - 1024.0

	d:drawImage(images[prevImage],
				pos,
				TwoFloats(1.0),
				col,
				1.0)
	d:drawImage(images[currentImage],
				pos,
				TwoFloats(1.0),
				col,
				fade)

	if scaleToggle then
		d:drawImage(images[currentImage],
				pos - (scale * 1024.0),
				TwoFloats(1.0 + scale),
				col,
				(1.0-(scale/4.0)) * scaleFade)
		d:drawImage(images[currentImage],
				pos - (scale2 * 1024.0),
				TwoFloats(1.0 + scale2),
				col,
				(1.0-(scale2/4.0)) * scaleFade)
		d:drawImage(images[currentImage],
				pos - (scale3 * 1024.0),
				TwoFloats(1.0 + scale3),
				col,
				(1.0-(scale3/4.0)) * scaleFade)
		d:drawImage(images[currentImage],
				pos - (scale4 * 1024.0),
				TwoFloats(1.0 + scale4),
				col,
				(1.0-(scale4/4.0)) * scaleFade)
	end
end

--------------------------------------------------------------------------------
--	Called when the user depresses a key or a joystick button.
function keyDown(action, keyPressed, keyModifier)

end

--------------------------------------------------------------------------------
--	Called when the user releases a key or a joystick button.
function keyUp(action, keyPressed, keyModifier)

end

--------------------------------------------------------------------------------
--	Called when the user depresses a mouse button.
function mouseDown(button, pos)
	
end

--------------------------------------------------------------------------------
--	Called when the user releases a mouse button.
function mouseUp(button, pos)

end

--------------------------------------------------------------------------------
--	Called when the user moves the mouse.
function mouseMove(pos)
	mousePos = pos
end

--------------------------------------------------------------------------------
--	Called when the user moves the mousewheel.
function mouseWheel(dir)

end

--------------------------------------------------------------------------------
--	Called when the user moves a joystick.
function joyMove(action, value, joystick, axis)

end

--------------------------------------------------------------------------------
--	Called each time the sequencer advances a step.
function nextStepCallback(step)

end
