//	KickSynth2.cpp - A kick synth based on MonophonicSynth.
//  ----------------------------------------------------------------------------
//	This file is part of 'NiallsAVLib', base code for any kind of audiovisual
//	apps.
//	Copyright (C) 2012  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#include "KickSynth2.h"
#include "MiscHelper.h"

//------------------------------------------------------------------------------
KickSynth2::KickSynth2():
start(0),
length(0.0f),
increment(8.0f/44100.0f),
x0(1.0f),
x1(0.0f),
hiIndex(0),
amplitude(1.0f)
{
	//hiCoeff[0] = 2.0f*sinf(3.1415926535897932384626433832795f*146.83238f/44100.0f);
	hiCoeff[0] = 2.0f*sinf(3.1415926535897932384626433832795f*55.0f/44100.0f);
	hiCoeff[1] = 2.0f*sinf(3.1415926535897932384626433832795f*110.0f/44100.0f);
	loCoeff = 2.0f*sinf(3.1415926535897932384626433832795f*50.0f/44100.0f);

	coeff = hiCoeff[0];
}

//------------------------------------------------------------------------------
KickSynth2::~KickSynth2()
{

}

//------------------------------------------------------------------------------
void KickSynth2::getAudio(float *buffer, int numSamples)
{
	int i;
	float amp;
	float tempf;

	if(!getActive())
		return;

	for(i=start;i<numSamples;++i)
	{
		//Count down the sound's timer.
		length -= increment;
		if(length <= 0.0f)
		{
			setActive(false);
			break;
		}

		//Generate the audio.
		x0 -= (coeff*x1);
		x1 += (coeff*x0);
		amp = fakeExp(length*0.5f);
		tempf = tanhf(x1 * amp * 3.14159265f * 1.2f);
		buffer[i*2] += tempf;
		buffer[(i*2)+1] += tempf;

		coeff = (loCoeff * (1.0f-length)) + (hiCoeff[hiIndex] * length);
		tempf = fabsf(x0)+fabsf(x1);
		if(tempf > 1.0f)
		{
			tempf = 1.0f/tempf;
			x0 *= tempf;
			x1 *= tempf;
		}
	}
	start = 0;
}

//------------------------------------------------------------------------------
void KickSynth2::triggerNote(float freq, int sample, float amp)
{
	if(!getActive())
	{
		start = sample;

		length = 2.0f;
		x0 = 1.0f;
		x1 = 0.0f;
		if(freq > 1680.0f)
			hiIndex = 1;
		else
			hiIndex = 0;
		//coeff = hiCoeff[hiIndex];

		setActive(true);
	}
}

//------------------------------------------------------------------------------
void KickSynth2::setParameter(const std::string& name, float value, bool direct)
{
	amplitude = value;
}
